@@ -16,6 +16,9 @@ This module is the UI controller for the customer, vendor, etc functions; it
 
 package LedgerSMB::Scripts::contact;
 
+use strict;
+use warnings;
+
 use LedgerSMB::Part;
 use LedgerSMB::Entity::Company;
 use LedgerSMB::Entity::Person;
@@ -29,13 +32,13 @@ use LedgerSMB::Entity::Bank;
 use LedgerSMB::Entity::Note;
 use LedgerSMB::Entity::User;
 use LedgerSMB::File;
+use LedgerSMB::Magic qw( EC_EMPLOYEE );
 use LedgerSMB::App_State;
 use LedgerSMB::Setting;
 use LedgerSMB::Template;
 use Try::Tiny;
 
-use strict;
-use warnings;
+use LedgerSMB::old_code qw(dispatch);
 
 #Plugins
 opendir(my $dh, 'lib/LedgerSMB/Entity/Plugins')
@@ -44,13 +47,14 @@ my @pluginmods = grep { /^[^.]/ && -f "LedgerSMB/Entity/Plugins/$_" } readdir($d
 closedir $dh;
 
 for (@pluginmods){
-    local ($!, $@);
+    local $! = undef;
+    local $@ = undef;
     my $do_ = "lib/LedgerSMB/Entity/Plugins/$_";
     if ( -e $do_ ) {
         unless ( do $do_ ) {
             if ($! or $@) {
-                print "Status: 500 Internal server error (contact.pm)\n\n";
-                warn "Failed to execute $do_ ($!): $@\n";
+                warn "\nFailed to execute $do_ ($!): $@\n";
+                die ( "Status: 500 Internal server error (contact.pm)\n\n" );
             }
         }
     }
@@ -78,7 +82,7 @@ control code
 
 sub get_by_cc {
     my ($request) = @_;
-    if ($request->{entity_class} == 3){
+    if ($request->{entity_class} == EC_EMPLOYEE){
         my $emp = LedgerSMB::Entity::Person::Employee->get_by_cc(
                             $request->{control_code}
         );
@@ -88,15 +92,15 @@ sub get_by_cc {
            LedgerSMB::Entity::Company->get_by_cc($request->{control_code});
     $entity ||=  LedgerSMB::Entity::Person->get_by_cc($request->{control_code});
     my ($company, $person) = (undef, undef);
-    { # pre-5.14 compatibility block
-        local ($@); # pre-5.14, do not die() in this block
-        if (eval {$entity->isa('LedgerSMB::Entity::Company')}){
-            $company = $entity;
-        } elsif (eval {$entity->isa('LedgerSMB::Entity::Person')}){
-            $person = $entity;
-        }
+
+    local $@ = undef;
+    if (eval {$entity->isa('LedgerSMB::Entity::Company')}){
+        $company = $entity;
+    } elsif (eval {$entity->isa('LedgerSMB::Entity::Person')}){
+        $person = $entity;
     }
-    _main_screen($request, $company, $person);
+
+    return _main_screen($request, $company, $person);
 }
 
 
@@ -112,7 +116,7 @@ of the company information.
 
 sub get {
     my ($request) = @_;
-    if ($request->{entity_class} && $request->{entity_class} == 3){
+    if ($request->{entity_class} && $request->{entity_class} == EC_EMPLOYEE){
         my $emp = LedgerSMB::Entity::Person::Employee->get(
                           $request->{entity_id}
         );
@@ -121,15 +125,15 @@ sub get {
     my $entity = LedgerSMB::Entity::Company->get($request->{entity_id});
     $entity ||= LedgerSMB::Entity::Person->get($request->{entity_id});
     my ($company, $person) = (undef, undef);
-    { # pre-5.14 compatibility block
-        local ($@); # pre-5.14, do not die() in this block
-        if (eval {$entity->isa('LedgerSMB::Entity::Company')}){
-            $company = $entity;
-        } elsif (eval {$entity->isa('LedgerSMB::Entity::Person')}){
-            $person = $entity;
-        }
+
+    local $@ = undef;
+    if (eval {$entity->isa('LedgerSMB::Entity::Company')}){
+        $company = $entity;
+    } elsif (eval {$entity->isa('LedgerSMB::Entity::Person')}){
+        $person = $entity;
     }
-    _main_screen($request, $company, $person);
+
+    return _main_screen($request, $company, $person);
 }
 
 
@@ -151,8 +155,8 @@ sub _main_screen {
        @DIVS = qw(credit address contact_info bank_act notes files);
        unshift @DIVS, 'company' if $company->{entity_id};
        unshift @DIVS, 'person' if $person->{entity_id};
-       no warnings 'uninitialized';
-       if ($person->{entity_id} && $person->{entity_class} == 3){
+       if ($person->{entity_id} && $person->{entity_class}
+                && $person->{entity_class} == EC_EMPLOYEE ){
           shift @DIVS;
           unshift @DIVS, 'employee', 'user', 'wage';
        }
@@ -162,7 +166,8 @@ sub _main_screen {
        my $employee = LedgerSMB::Entity::Person::Employee->get($entity_id);
        $person = $employee if $employee;
        $user = LedgerSMB::Entity::User->get($entity_id);
-    } elsif (defined $person->{entity_class} && $person->{entity_class} == 3) {
+    } elsif (defined $person->{entity_class}
+                && $person->{entity_class} == EC_EMPLOYEE ) {
        @DIVS = ('employee');
     } else {
        @DIVS = qw(company person);
@@ -270,7 +275,7 @@ sub _main_screen {
 
     for my $var (\@ar_ap_acc_list, \@cash_acc_list, \@discount_acc_list){
         for my $ref (@$var){
-            $ref->{description} ||= "";
+            $ref->{description} ||= '';
             $ref->{text} = "$ref->{accno}--$ref->{description}";
         }
     }
@@ -284,7 +289,7 @@ sub _main_screen {
     }
 
     my @location_class_list =
-       grep { $_->{id} < 4 }
+       grep { $_->{class} =~ m/^(?:Billing|Sales|Shipping)$/ }
             LedgerSMB->call_procedure(funcname => 'location_list_class');
 
     my @business_types =
@@ -309,11 +314,11 @@ sub _main_screen {
          value => 3} if $credit_act->{id};
     ;
 
-    { # pre-5.14 compatibility block
-        local ($@); # pre-5.14, do not die() in this block
-        $request->close_form() if eval {$request->can('close_form')};
-        $request->open_form() if eval {$request->can('close_form')};
-    }
+
+    local $@ = undef;
+    $request->close_form() if eval {$request->can('close_form')};
+    $request->open_form() if eval {$request->can('close_form')};
+
     opendir(my $dh2, 'UI/Contact/plugins') || die "can't opendir plugins directory: $!";
     my @plugins = grep { /^[^.]/ && -f "UI/Contact/plugins/$_" } readdir($dh2);
     closedir $dh2;
@@ -337,7 +342,7 @@ sub _main_screen {
     my $roles;
     $roles = $user->list_roles if $user;
 
-    $template->render({
+    return $template->render({
                      DIVS => \@DIVS,
                 DIV_LABEL => \%DIV_LABEL,
              entity_class => $entity_class,
@@ -397,7 +402,7 @@ sub save_employee {
                            );
         ($request->{control_code}) = values %$ref;
     }
-    $request->{entity_class} = 3;
+    $request->{entity_class} = EC_EMPLOYEE ;
     $request->{ssn} = $request->{personal_id} if defined $request->{personal_id};
     $request->{control_code} = $request->{employeenumber} if defined $request->{employeenumber};
     $request->{employeenumber} ||= $request->{control_code};
@@ -405,7 +410,7 @@ sub save_employee {
     my $employee = LedgerSMB::Entity::Person::Employee->new(%$request);
     $request->{target_div} = 'employee_div';
     $employee->save;
-    _main_screen($request, undef, $employee);
+    return _main_screen($request, undef, $employee);
 }
 
 =item generate_control_code
@@ -426,7 +431,7 @@ sub generate_control_code {
     $person = $request if $request->{entity_id} and $request->{first_name};
     ($person, $company) = ($request, $request)
         unless $person or $company;
-    _main_screen($request, $company, $person);
+    return _main_screen($request, $company, $person);
 }
 
 =item dispatch_legacy
@@ -463,21 +468,22 @@ sub dispatch_legacy {
     } else {
        $request->error($request->{_locale}->text('Unsupported account type'));
     }
-    our $dispatch =
+
+    my $dispatch =
     {
-        add_transaction  => {script => "bin/$aa.pl",
+        add_transaction  => {script => "$aa.pl",
                                data => {"${cv}_id" => $request->{credit_id}},
                             },
-        add_invoice      => {script => "bin/$inv.pl",
+        add_invoice      => {script => "$inv.pl",
                                data => {"${cv}_id" => $request->{credit_id}},
                             },
-        add_order        => {script => 'bin/oe.pl',
+        add_order        => {script => 'oe.pl',
                                data => {"${cv}_id" => $request->{credit_id},
                                             type   => $otype,
                                                vc  => $cv,
                                        },
                             },
-        rfq              => {script => 'bin/oe.pl',
+        rfq              => {script => 'oe.pl',
                                data => {"${cv}_id" => $request->{credit_id},
                                             type   => $qtype,
                                                vc  => $cv,
@@ -486,39 +492,13 @@ sub dispatch_legacy {
 
     };
 
-    # set up environment for call to legacy code
-    our $form = new Form;
-    our %myconfig = ();
-    our $locale = $request->{_locale};
-    %myconfig = %{$request->{_user}};
-    $form->{stylesheet} = $myconfig{stylesheet};
-
-    for (keys %{$dispatch->{$request->{action}}->{data}}){
-        $form->{$_} = $dispatch->{$request->{action}}->{data}->{$_};
-    }
-
-    if (my $cpid = fork()) {
-        waitpid $cpid, 0;
-    }
-    else {
-        # We need a 'try' block here to prevent errors being thrown in
-        # the inner block from escaping out of the block and missing
-        # the 'exit' below.
-        try {
-            my $script = $dispatch->{$request->{action}}{script};
-            $form->{script} = $script;
-            $form->{action} = 'add';
-            $form->{dbh} = $request->{dbh};
-            $form->{script} =~ s|.*/||;
-            { no strict; no warnings 'redefine'; do $script; }
-            { no warnings;
-              # Suppress 'only referenced once' warnings
-              $lsmb_legacy::form = $form;
-              $lsmb_legacy::locale = $locale; }
-            "lsmb_legacy"->can($form->{action})->();
-        };
-        exit;
-    }
+    my $entry = $dispatch->{$request->{action}};
+    return dispatch($entry->{script},
+                    'add',
+                    { %{$entry->{data}},
+                      script => $entry->{script},
+                      action => 'add',
+                      dbh => $request->{dbh} });
 }
 
 =item add_transaction
@@ -529,7 +509,7 @@ Dispatches to the Add (AR or AP as appropriate) transaction screen.
 
 sub add_transaction {
     my $request = shift @_;
-    dispatch_legacy($request);
+    return dispatch_legacy($request);
 }
 
 =item add_invoice
@@ -540,7 +520,7 @@ Dispatches to the (sales or vendor, as appropriate) invoice screen.
 
 sub add_invoice {
     my $request = shift @_;
-    dispatch_legacy($request);
+    return dispatch_legacy($request);
 }
 
 =item add_order
@@ -551,7 +531,7 @@ Dispatches to the sales/purchase order screen.
 
 sub add_order {
     my $request = shift @_;
-    dispatch_legacy($request);
+    return dispatch_legacy($request);
 }
 
 =item rfq
@@ -562,7 +542,7 @@ Dispatches to the quotation/rfq screen
 
 sub rfq {
     my $request = shift @_;
-    dispatch_legacy($request);
+    return dispatch_legacy($request);
 }
 
 =item add
@@ -574,7 +554,7 @@ This method creates a blank screen for entering a company's information.
 sub add {
     my ($request) = @_;
     $request->{target_div} = 'company_div';
-    _main_screen($request, $request);
+    return _main_screen($request, $request);
 }
 
 =item save_company
@@ -595,7 +575,7 @@ sub save_company {
     $request->{name} ||= $request->{legal_name};
     my $company = LedgerSMB::Entity::Company->new(%$request);
     $request->{target_div} = 'credit_div';
-    _main_screen($request, $company->save);
+    return _main_screen($request, $company->save);
 }
 
 =item save_person
@@ -606,7 +586,7 @@ Saves a person and moves on to the next screen
 
 sub save_person {
     my ($request) = @_;
-    if ($request->{entity_class} == 3){
+    if ($request->{entity_class} == EC_EMPLOYEE ){
         $request->{dob} = $request->{birthdate} if $request->{birthdate};
        return save_employee($request);
     }
@@ -622,7 +602,7 @@ sub save_person {
     );
     $request->{target_div} = 'credit_div';
     $person->save;
-    _main_screen($request, undef, $person);
+    return _main_screen($request, undef, $person);
 }
 
 =item save_credit($request)
@@ -636,7 +616,6 @@ sub save_credit {
     my ($request) = @_;
     $request->{target_div} = 'credit_div';
     my $company;
-    my @taxes;
 
     if (!$request->{ar_ap_account_id}){
           $request->error(
@@ -656,7 +635,7 @@ sub save_credit {
         $credit = $credit->save();
         $request->{meta_number} = $credit->{meta_number};
     }
-    get($request);
+    return get($request);
 }
 
 =item save_credit_new($request)
@@ -669,7 +648,7 @@ This inserts a new credit account.
 sub save_credit_new {
     my ($request) = @_;
     delete $request->{id};
-    save_credit($request);
+    return save_credit($request);
 }
 
 =item save_location
@@ -693,7 +672,7 @@ sub save_location {
     # Assumption alert!  Assuming additional addresses share a city, state
     # and country more often than not -- CT
     delete $request->{"$_"} for (qw(line_one line_two line_three mail_code));
-    get($request);
+    return get($request);
 
 }
 
@@ -707,7 +686,7 @@ overwriting existing locations.
 sub save_new_location {
     my ($request) = @_;
     delete $request->{location_id};
-    save_location($request);
+    return save_location($request);
 }
 
 =item edit
@@ -717,7 +696,7 @@ This is a synonym of get() which is preferred to use for editing operations.
 =cut
 
 sub edit {
-    get (@_);
+    return get (@_);
 }
 
 =item delete_location
@@ -739,7 +718,7 @@ sub delete_location {
 
     $request->{target_div} = 'address_div';
     $request->{credit_id}=$credit_id;
-    get($request);
+    return get($request);
 }
 
 =item save_contact
@@ -761,7 +740,7 @@ sub save_contact {
     $request->{target_div} = 'contact_info_div';
     delete $request->{description};
     delete $request->{contact};
-    get($request);
+    return get($request);
 }
 
 =item save_contact_new
@@ -775,7 +754,7 @@ sub save_contact_new {
     delete $request->{contact_id};
     delete $request->{old_contact};
 
-    save_contact($request);
+    return save_contact($request);
 }
 
 =item delete_contact
@@ -789,7 +768,7 @@ sub delete_contact {
     my ($request) = @_;
     LedgerSMB::Entity::Contact::delete($request);
     $request->{target_div} = 'contact_info_div';
-    get($request);
+    return get($request);
 }
 
 =item delete_bank_account
@@ -807,7 +786,7 @@ sub delete_bank_account{
     my ($request) = @_;
     LedgerSMB::Entity::Bank->get($request->{id})->delete;
     $request->{target_div} = 'bank_act_div';
-    get($request);
+    return get($request);
 }
 
 =item save_bank_account
@@ -821,7 +800,7 @@ sub save_bank_account {
     my $bank = LedgerSMB::Entity::Bank->new(%$request);
     $bank->save;
     $request->{target_div} = 'bank_act_div';
-    get($request);
+    return get($request);
 }
 
 =item save_notes($request)
@@ -835,7 +814,7 @@ sub save_notes {
     my ($request) = @_;
     my $note = LedgerSMB::Entity::Note->new(%$request);
     $note->save;
-    get($request);
+    return get($request);
 }
 
 =item get_pricelist
@@ -859,7 +838,7 @@ sub get_pricelist {
                 format => uc($request->{format} || 'HTML'),
                 locale => $request->{_locale},
     );
-    $template->render($request);
+    return $template->render($request);
 }
 
 
@@ -875,15 +854,12 @@ and the description is a full text search.
 
 sub save_pricelist {
     my ($request) = @_;
-    use LedgerSMB::App_State;
     use LedgerSMB::DBObject::Pricelist;
 
     my $count = $request->{rowcount_pricematrix};
 
     my $pricelist = LedgerSMB::DBObject::Pricelist->new({base => $request});
     my @lines;
-    my $redirect_to_selection = 0;
-    my $psearch;
 
     # Search and populate
     if (defined $request->{"int_partnumber_tfoot_$count"}
@@ -919,13 +895,7 @@ sub save_pricelist {
 
     $pricelist->save(\@lines);
 
-    # Return to UI
-    if (!$redirect_to_selection){
-        get_pricelist($request);
-    } else {
-        $request->{search_redirect} = 'pricelist_search_handle';
-        $psearch->render($request);
-   }
+    return get_pricelist($request);
 }
 
 
@@ -942,7 +912,7 @@ sub delete_pricelist {
     $pricelist->delete;
 
     # Return to UI
-    get_pricelist($request);
+    return get_pricelist($request);
 }
 
 =item create_user
@@ -971,7 +941,7 @@ sub create_user {
            $request->{pls_import} = 1;
        }
     }
-    get($request);
+    return get($request);
 }
 
 =item reset_password
@@ -987,7 +957,7 @@ sub reset_password {
        my $user = LedgerSMB::Entity::User->new(%$request);
        $user->reset_password($request->{password});
     }
-    get($request);
+    return get($request);
 }
 
 =item save_roles
@@ -1010,7 +980,7 @@ sub save_roles {
        }
        $user->save_roles($roles);
     }
-    get($request);
+    return get($request);
 }
 
 =back
